/**
 * @fileoverview No unused React component methods
 * @author Nir
 */
"use strict";

//------------------------------------------------------------------------------
// Rule Definition
//------------------------------------------------------------------------------

function isReactComponent (node, context) {
  if (!node.superClass) {
    return false;
  }
  return /^(React\.)?(Pure)?Component$/.test(context.getSourceCode().getText(node.superClass));
}

function getClassNameScope(node, context){
  let scope = context.getScope(node);
  while(scope){
    if(scope.type === 'class') {
      console.log('*****************blcok',scope.block.id.name);
      return;
    }
    scope = scope.upper;
  }
}


module.exports = {
    meta: {
        docs: {
            description: "No unused React component methods",
            category: "Fill me in",
            recommended: false
        },
        fixable: null,  // or "code" or "whitespace"
        schema: [
            // fill in your schema
        ]
    },

    create: function(context) {
      let methods = {};

      function addMethod(methodName, node){
        if(methods[methodName]) {
          methods[methodName].node = node;
        } else {
          methods[methodName] = { node, methodName, isUsed: false};
        }
      }
        // variables should be defined here

        //----------------------------------------------------------------------
        // Helpers
        //----------------------------------------------------------------------

        // any helper functions should go here or else delete this section

        //----------------------------------------------------------------------
        // Public
        //----------------------------------------------------------------------
        return {
  
            // 'ClassDeclaration[superClass.name="Component"]': (node) => {
            //   console.log('nirrrrrrrrrrrrrrrrrrrrrrrrr');
            // },
            'MethodDefinition': (node) => {
              addMethod(node.key.name, node);
            },
            'ClassProperty': (node) => {
              addMethod(node.key.name, node);
            },
            'MemberExpression': (node) => {
                if(node.object.type === 'ThisExpression'){
                  const methodName =  node.property.name;
                  if(!methods[methodName]) {
                    addMethod(methodName);
                  }
                  methods[methodName].isUsed = true;
                }
            },
            'ClassDeclaration:exit': (node) => {
              if(isReactComponent(node, context)){
                console.log('class for:', node.id.name, ' ', Object.keys(methods).filter(key => !methods[key].isUsed).map(method=> method.methodName));
                const unUsedMethods = Object.keys(methods).filter(key => !methods[key].isUsed);
                unUsedMethods.forEach(method => context.report({
                  node: method.node,
                  message: 'unused class method {{methodName}}',
                  data:{
                    methodName : method.methodName
                  }
                }));
              }
                methods = {};
            }
        };
    }
};
